@@ -1,20 +1,19 @@
+from blacksheep.server.files import ServeFilesOptions
 import logging
 from typing import (
     Any,
     Awaitable,
     Callable,
+    Iterable,
     List,
     Optional,
+    Sequence,
+    Set,
     Tuple,
     Type,
-    Sequence,
+    Union,
 )
 
-from guardpost.asynchronous.authentication import AuthenticationStrategy
-from guardpost.asynchronous.authorization import AuthorizationStrategy
-from guardpost.authorization import Policy, UnauthorizedError
-from rodi import Container, Services
-
 from blacksheep.baseapp import BaseApplication
 from blacksheep.common.files.asyncfs import FilesHandler
 from blacksheep.contents import ASGIContent
@@ -33,11 +32,17 @@ from blacksheep.server.authorization import (
 )
 from blacksheep.server.bindings import ControllerParameter
 from blacksheep.server.controllers import router as controllers_router
-from blacksheep.server.files.dynamic import ServeFilesOptions, serve_files_dynamic
+from blacksheep.server.cors import CORSPolicy, CORSStrategy, get_cors_middleware
+from blacksheep.server.files.dynamic import serve_files_dynamic
 from blacksheep.server.normalization import normalize_handler, normalize_middleware
 from blacksheep.server.resources import get_resource_file_content
 from blacksheep.server.routing import RegisteredRoute, Router, RoutesRegistry
 from blacksheep.utils import ensure_bytes, join_fragments
+from guardpost.asynchronous.authentication import AuthenticationStrategy
+from guardpost.asynchronous.authorization import AuthorizationStrategy
+from guardpost.authorization import Policy, UnauthorizedError
+from guardpost.common import AuthenticatedRequirement
+from rodi import Container, Services
 
 __all__ = ("Application",)
 
@@ -97,6 +102,14 @@ class RequiresServiceContainerError(ApplicationStartupError):
         self.details = details
 
 
+class ApplicationAlreadyStartedCORSError(TypeError):
+    def __init__(self) -> None:
+        super().__init__(
+            "The application is already running, configure CORS rules "
+            "before starting the application"
+        )
+
+
 class Application(BaseApplication):
     def __init__(
         self,
@@ -119,11 +132,10 @@ class Application(BaseApplication):
         self._service_provider: Optional[Services] = None
         self.debug = debug
         self.middlewares: List[Callable[..., Awaitable[Response]]] = []
-        self.access_logger = None
-        self.logger = None
         self._default_headers: Optional[Tuple[Tuple[str, str], ...]] = None
         self._middlewares_configured = False
         self.resources = resources
+        self._cors_strategy: Optional[CORSStrategy] = None
         self._authentication_strategy: Optional[AuthenticationStrategy] = None
         self._authorization_strategy: Optional[AuthorizationStrategy] = None
         self.on_start = ApplicationEvent(self)
@@ -150,6 +162,97 @@ class Application(BaseApplication):
     def default_headers(self, value: Optional[Tuple[Tuple[str, str], ...]]) -> None:
         self._default_headers = tuple(value) if value else None
 
+    @property
+    def cors(self) -> CORSStrategy:
+        if not self._cors_strategy:
+            raise TypeError(
+                "CORS settings are not initialized for the application. "
+                + "Use `app.use_cors()` method before using this property."
+            )
+        return self._cors_strategy
+
+    def use_cors(
+        self,
+        *,
+        allow_methods: Union[None, str, Iterable[str]] = None,
+        allow_headers: Union[None, str, Iterable[str]] = None,
+        allow_origins: Union[None, str, Iterable[str]] = None,
+        allow_credentials: bool = False,
+        max_age: int = 5,
+        expose_headers: Union[None, str, Iterable[str]] = None,
+    ) -> CORSStrategy:
+        """
+        Enables CORS for the application, specifying the default rules to be applied
+        for all request handlers.
+        """
+        if self.started:
+            raise ApplicationAlreadyStartedCORSError()
+        self._cors_strategy = CORSStrategy(
+            CORSPolicy(
+                allow_methods=allow_methods,
+                allow_headers=allow_headers,
+                allow_origins=allow_origins,
+                allow_credentials=allow_credentials,
+                max_age=max_age,
+                expose_headers=expose_headers,
+            ),
+            self.router,
+        )
+
+        # Note: the following is a no-op request handler, necessary to activate handling
+        # of OPTIONS preflight requests.
+        # However, preflight requests are handled by the CORS middleware. This is to
+        # stop the chain of middlewares and prevent extra logic from executing for
+        # preflight requests (e.g. authentication logic)
+        @self.router.options("*")
+        async def options_handler(request):
+            return Response(404)
+
+        # User defined catch-all OPTIONS request handlers are not supported when the
+        # built-in CORS handler is used.
+        return self._cors_strategy
+
+    def add_cors_policy(
+        self,
+        policy_name,
+        *,
+        allow_methods: Union[None, str, Iterable[str]] = None,
+        allow_headers: Union[None, str, Iterable[str]] = None,
+        allow_origins: Union[None, str, Iterable[str]] = None,
+        allow_credentials: bool = False,
+        max_age: int = 5,
+        expose_headers: Union[None, str, Iterable[str]] = None,
+    ) -> None:
+        """
+        Configures a set of CORS rules that can later be applied to specific request
+        handlers, by name.
+
+        The CORS policy can then be associated to specific request handlers,
+        using the instance of `CORSStrategy` as a function decorator:
+
+        @app.cors("example")
+        @app.route("/")
+        async def foo():
+            ....
+        """
+        if self.started:
+            raise ApplicationAlreadyStartedCORSError()
+
+        if not self._cors_strategy:
+            self.use_cors()
+
+        self._cors_strategy.add_policy(
+            policy_name,
+            CORSPolicy(
+                allow_methods=allow_methods,
+                allow_headers=allow_headers,
+                allow_origins=allow_origins,
+                allow_credentials=allow_credentials,
+                max_age=max_age,
+                expose_headers=expose_headers,
+            ),
+        )
+
     def use_authentication(
         self, strategy: Optional[AuthenticationStrategy] = None
     ) -> AuthenticationStrategy:
@@ -178,11 +281,10 @@ class Application(BaseApplication):
 
         if strategy.default_policy is None:
             # by default, a default policy is configured with no requirements,
-            # meaning that request handlers allow anonymous users, unless
-            # specified otherwise
-            # this can be modified, by adding a requirement to the default
-            # policy
+            # meaning that request handlers allow anonymous users by default, unless
+            # they are decorated with @auth()
             strategy.default_policy = Policy("default")
+            strategy.add(Policy("authenticated").add(AuthenticatedRequirement()))
 
         self._authorization_strategy = strategy
         self.exceptions_handlers[
@@ -204,8 +306,68 @@ class Application(BaseApplication):
 
         return decorator
 
-    def serve_files(self, options: ServeFilesOptions):
-        serve_files_dynamic(self.router, self.files_handler, options)
+    def serve_files(
+        self,
+        source_folder: str,
+        *,
+        discovery: bool = False,
+        cache_time: int = 10800,
+        extensions: Optional[Set[str]] = None,
+        root_path: str = "",
+        index_document: Optional[str] = "index.html",
+        fallback_document: Optional[str] = None,
+        allow_anonymous: bool = True,
+    ):
+        """
+        Configures dynamic file serving from a given folder, relative to the server cwd.
+
+        Parameters:
+            source_folder (str): Path to the source folder containing static files.
+            extensions: The set of files extensions to serve.
+            discovery: Whether to enable file discovery, serving HTML pages for folders.
+            cache_time: Controls the Cache-Control Max-Age in seconds for static files.
+            root_path: Path prefix used for routing requests.
+            For example, if set to "public", files are served at "/public/*".
+            allow_anonymous: Whether to enable anonymous access to static files, true by
+            default.
+            index_document: The name of the index document to display, if present,
+            in folders. Requests for folders that contain a file with matching produce
+            a response with this document.
+            fallback_document: Optional file name, for a document to serve when a
+            response would be otherwise 404 Not Found; e.g. use this to serve SPA that
+            use HTML5 History API for client side routing.
+        """
+        if isinstance(source_folder, ServeFilesOptions):
+            # deprecated class, will be removed in the next version
+            from typing import cast
+
+            deprecated_arg = cast(ServeFilesOptions, source_folder)
+            deprecated_arg.validate()
+            serve_files_dynamic(
+                self.router,
+                self.files_handler,
+                str(deprecated_arg.source_folder),
+                discovery=deprecated_arg.discovery,
+                cache_time=deprecated_arg.cache_time,
+                extensions=deprecated_arg.extensions,
+                root_path=deprecated_arg.root_path,
+                index_document=deprecated_arg.index_document,
+                fallback_document=deprecated_arg.fallback_document,
+                anonymous_access=deprecated_arg.allow_anonymous,
+            )
+            return
+        serve_files_dynamic(
+            self.router,
+            self.files_handler,
+            source_folder,
+            discovery=discovery,
+            cache_time=cache_time,
+            extensions=extensions,
+            root_path=root_path,
+            index_document=index_document,
+            fallback_document=fallback_document,
+            anonymous_access=allow_anonymous,
+        )
 
     def _apply_middlewares_in_routes(self):
         for route in self.router:
@@ -341,6 +503,9 @@ class Application(BaseApplication):
                 0, get_authentication_middleware(self._authentication_strategy)
             )
 
+        if self._cors_strategy:
+            self.middlewares.insert(0, get_cors_middleware(self._cors_strategy))
+
         if self._default_headers:
             self.middlewares.insert(
                 0, get_default_headers_middleware(self._default_headers)
